@@ -1,4 +1,4 @@
-# Copyright (C) 2019-2021 Clémence Frioux & Arnaud Belcour - Inria Dyliss - Pleiade
+# Copyright (C) 2019-2022 Clémence Frioux & Arnaud Belcour - Inria Dyliss - Pleiade
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -23,6 +23,7 @@ import requests
 import simplejson
 import shutil
 import sys
+import traceback
 
 from Bio.SeqRecord import SeqRecord
 from Bio import SeqFeature as sf
@@ -155,29 +156,38 @@ def create_GO_namespaces_alternatives(gobasic_file=None):
     go_basic_obo_url = 'http://purl.obolibrary.org/obo/go/go-basic.obo'
 
     if gobasic_file:
-        if is_valid_file(gobasic_file):
-            go_ontology = pronto.Ontology(gobasic_file)
-        else:
+        if not is_valid_file(gobasic_file):
             logger.critical(gobasic_file + ' file is not available, emapper2gbk will download the GO ontology and create it.')
             response = requests.get(go_basic_obo_url, stream=True)
             with open(gobasic_file, 'wb') as go_basic_file_write:
                 shutil.copyfileobj(response.raw, go_basic_file_write)
-            go_ontology = pronto.Ontology(go_basic_obo_url)
     else:
-        go_ontology = pronto.Ontology(go_basic_obo_url)
+        gobasic_file = go_basic_obo_url
+
+    try:
+        go_ontology = pronto.Ontology(gobasic_file)
+    except SyntaxError as syntax_error:
+        traceback.print_exc()
+        logger.critical('/!\\ It seems that the obo fie (' + gobasic_file + ') has a syntax error.')
+        logger.critical('You can try other obo file like the one from the github of gene Ontology (which can contains the fix for the syntax error): https://github.com/geneontology/go-ontology/blob/104252cc533a83248ed4df2caefd434e6eb8b531/src/ontology/go-edit.obo.')
+        logger.critical('Or the obo file from the test folder of emapper2gbk (https://github.com/AuReMe/emapper2gbk/blob/master/tests/go-basic.obo).')
+        sys.exit(1)
 
     # For each GO terms look to the namespaces associated with them.
     go_namespaces = {}
     for go_term in go_ontology:
         if 'GO:' in go_term:
-            go_namespaces[go_term] = go_ontology[go_term].namespace
+            go = go_ontology[go_term]
+            go_namespaces[go_term] = go.namespace
 
     # For each GO terms look if there is an alternative ID fo them.
     go_alternative = {}
     for go_term in go_ontology:
-        if go_ontology[go_term].alternate_ids != frozenset():
-            for go_alt in go_ontology[go_term].alternate_ids:
-                go_alternative[go_alt] = go_term
+        if 'GO:' in go_term:
+            go = go_ontology[go_term]
+            if go.alternate_ids != frozenset():
+                for go_alt in go.alternate_ids:
+                    go_alternative[go_alt] = go_term
 
     return go_namespaces, go_alternative
 
@@ -196,11 +206,6 @@ def create_taxonomic_data(species_name):
     species_informations = {}
 
     compatible_species_name = species_name.replace('/', '_')
-    species_informations['description'] = compatible_species_name + ' genome'
-    species_informations['organism'] = compatible_species_name
-    species_informations['keywords'] = [compatible_species_name]
-
-    species_name_url = species_name.replace(' ', '%20')
 
     if species_name == "bacteria":
         species_informations = {'db_xref': 'taxon:2', 'scientificName': 'Bacteria', 'commonName': 'eubacteria', 'formalName': 'false', 'rank': 'superkingdom', 'data_file_division': 'PRO', 'geneticCode': '11', 'submittable': 'false', 'description': 'bacteria genome', 'organism': 'bacteria', 'keywords': ['bacteria']} 
@@ -213,22 +218,42 @@ def create_taxonomic_data(species_name):
     elif species_name == 'cellular organisms':
         species_informations = {'db_xref': 'taxon:131567', 'scientificName': 'cellular organisms', 'formalName': 'false', 'rank': 'no rank', 'division': 'UNC', 'geneticCode': '1', 'submittable': 'false'}
     else:
-        url = 'https://www.ebi.ac.uk/ena/data/taxonomy/v1/taxon/scientific-name/' + species_name_url
+        taxons = species_name.split(";")
+        taxon_found = False
 
-        try:
-            response = requests.get(url)
-        except requests.exceptions.ConnectionError:
-            logger.critical('/!\\ No internet connection, check the connection or use the --ete option (if you have the NCBITaxa database already downloaded).')
-            return None
+        for index, taxon in reversed(list(enumerate(taxons))):
+            if not taxon or taxon == '':
+                continue
+            compatible_species_name = taxon.replace('/', '_')
+            species_name_url = taxon.replace(' ', '%20')
+            url = 'https://www.ebi.ac.uk/ena/data/taxonomy/v1/taxon/scientific-name/' + species_name_url
 
-        # Check if there is taxonomy information in the EBI response JSON.
-        try:
-            temp_species_informations = response.json()[0]
-        except simplejson.errors.JSONDecodeError:
-            logger.critical('/!\\ Error with {} this taxa has not been found in https://www.ebi.ac.uk/ena/data/taxonomy/v1/taxon/scientific-name/'.format(species_name))
-            logger.critical('/!\\ Check the name of the taxa and its presence in the EBI taxonomy database.')
-            logger.critical('/!\\ No genbank will be created for {}.'.format(species_name))
-            return None
+            try:
+                response = requests.get(url)
+            except requests.exceptions.ConnectionError:
+                logger.critical('/!\\ No internet connection, check the connection or use the --ete option (if you have the NCBITaxa database already downloaded).')
+                return None
+
+            # Check if there is taxonomy information in the EBI response JSON.
+            try:
+                temp_species_informations = response.json()
+                taxon_found = True
+            except simplejson.errors.JSONDecodeError:
+                if index==len(taxons):
+                    logger.critical('/!\\ Error with {0} this taxa has not been found in https://www.ebi.ac.uk/ena/data/taxonomy/v1/taxon/scientific-name/'.format(species_name))
+                    logger.critical('/!\\ Check the name of the taxa and its presence in the EBI taxonomy database.')
+                    logger.critical('/!\\ No genbank will be created for {0}.'.format(taxon))
+                    return None
+                else:
+                    continue
+
+            if temp_species_informations == []:
+                logger.critical('/!\\ Error with {0} this taxa has not been found in https://www.ebi.ac.uk/ena/data/taxonomy/v1/taxon/scientific-name/'.format(taxon))
+                continue
+            if taxon_found is True:
+                break
+
+        temp_species_informations = temp_species_informations[0]
 
         for temp_species_information in temp_species_informations:
             if temp_species_information == 'lineage':
@@ -240,6 +265,10 @@ def create_taxonomic_data(species_name):
             else:
                 species_informations[temp_species_information] = temp_species_informations[temp_species_information]
 
+    species_informations['description'] = compatible_species_name + ' genome'
+    species_informations['organism'] = compatible_species_name
+    species_informations['keywords'] = [compatible_species_name]
+
     return species_informations
 
 
@@ -256,22 +285,29 @@ def create_taxonomic_data_ete(species_name):
         species_informations (dict): dictionary containing information about species
     """
     species_informations = {}
+    taxons = species_name.split(";")
+    for index, taxon in reversed(list(enumerate(taxons))):
+        if not taxon:
+            continue
+        compatible_species_name = taxon.replace('/', '_')
+        species_informations['description'] = compatible_species_name + ' genome'
+        species_informations['organism'] = compatible_species_name
+        species_informations['keywords'] = [compatible_species_name]
 
-    compatible_species_name = species_name.replace('/', '_')
-    species_informations['description'] = compatible_species_name + ' genome'
-    species_informations['organism'] = compatible_species_name
-    species_informations['keywords'] = [compatible_species_name]
-
-    ncbi = NCBITaxa()
-    species_taxids = ncbi.get_name_translator([species_name])
-    if species_name in species_taxids:
-        species_taxid = species_taxids[species_name][-1]
-        species_informations['db_xref'] = 'taxon:' + str(species_taxid)
-    else:
-        logger.critical('/!\\ Error with {} this taxa has not been found in ete3 NCBITaxa Database'.format(species_name))
-        logger.critical('/!\\ Check the name of the taxa and its presence in the NCBITaxa database.')
-        logger.critical('/!\\ No genbank will be created for {}.'.format(species_name))
-        return None
+        ncbi = NCBITaxa()
+        species_taxids = ncbi.get_name_translator([taxon])
+        if taxon in species_taxids:
+            species_taxid = species_taxids[taxon][-1]
+            species_informations['db_xref'] = 'taxon:' + str(species_taxid)
+            break
+        else:
+            if index ==0:
+                logger.critical('/!\\ Error with {} this taxa has not been found in ete3 NCBITaxa Database'.format(taxon))
+                logger.critical('/!\\ Check the name of the taxa and its presence in the NCBITaxa database.')
+                logger.critical('/!\\ No genbank will be created for {}.'.format(taxon))
+                return None
+            else:
+                continue
 
     return species_informations
 
@@ -377,15 +413,25 @@ def read_annotation(eggnog_outfile:str):
         # Assign the headers
         annotation_data.columns = headers_row
         if 'query_name' in annotation_data.columns:
+            # Check if the gene IDs are numeric, if yes add 'gene_' in front of them.
+            numeric_row_dataframe = pd.to_numeric(annotation_data['query_name'], errors='coerce').notnull()
+            if bool(numeric_row_dataframe.any()) is True:
+                annotation_data.loc[numeric_row_dataframe, 'query_name'] = 'gene_' + annotation_data.loc[numeric_row_dataframe, 'query_name']
             annotation_dict = annotation_data.set_index('query_name')[to_extract_annotations].to_dict('index')
         # 'query' added for compatibility with eggnog-mapper 2.1.2
         elif 'query' in annotation_data.columns:
+            numeric_row_dataframe = pd.to_numeric(annotation_data['query'], errors='coerce').notnull()
+            if bool(numeric_row_dataframe.any()) is True:
+                annotation_data.loc[numeric_row_dataframe, 'query'] = 'gene_' + annotation_data.loc[numeric_row_dataframe, 'query']
             annotation_dict = annotation_data.set_index('query')[to_extract_annotations].to_dict('index')
         for key in annotation_dict:
             yield key, annotation_dict[key]
 
 
-def create_cds_feature(id_gene, start_position, end_position, strand, annot, go_namespaces, go_alternatives, gene_protein_seq, gff_extracted_annotations=None):
+def create_cds_feature(id_gene, start_position, end_position,
+                        strand, annot, go_namespaces, go_alternatives,
+                        gene_protein_seq, gff_extracted_annotations=None,
+                        location_exons=None):
     """ Create Biopython CDS feature from gene ID, gene positions, gene sequecne and gene annotations.
 
     Args:
@@ -398,14 +444,20 @@ def create_cds_feature(id_gene, start_position, end_position, strand, annot, go_
         go_alternatives (dict): dictionary of GO terms alternatives ID (key: GO Term ID, value: alternatives GO Term associated to GO Term)
         gene_protein_seq (dict): dictionary of protein sequence associated to genes (key: gene id, value: sequence)
         gff_extracted_annotations (dict): dictionary of annotation to copy from the GFF file into the Genbank file.
+        location_exons (list): list of FeatureLocation containing exons positions for the CDS, None if no exon position or if not in gff type gmove.
 
     Returns:
         new_feature_cds (Bio.SeqFeature.SeqFeature): New SeqFeature containing the informations provided
     """
-    new_feature_cds = sf.SeqFeature(sf.FeatureLocation(start_position,
-                                                        end_position,
-                                                        strand),
-                                                    type="CDS")
+    if location_exons and len(location_exons)>=2:
+        exon_feature_locations = sf.CompoundLocation(location_exons, operator='join')
+
+        new_feature_cds = sf.SeqFeature(exon_feature_locations, type='CDS')
+    else:
+        new_feature_cds = sf.SeqFeature(sf.FeatureLocation(start_position,
+                                                            end_position,
+                                                            strand),
+                                                                    type="CDS")
 
     new_feature_cds.qualifiers['locus_tag'] = id_gene
 
